@@ -26,6 +26,8 @@ PYBIND11_MODULE(_gridpy, m) {
 
     m.doc() = "PowSyBl Python API";
 
+    py::register_exception<gridpy::GridPyError>(m, "GridPyError");
+
     m.def("set_debug_mode", &gridpy::setDebugMode, "Set debug mode");
 
     m.def("get_version_table", &gridpy::getVersionTable, "Get an ASCII table with all PowSybBl modules version");
@@ -42,15 +44,23 @@ PYBIND11_MODULE(_gridpy, m) {
     m.def("update_connectable_status", &gridpy::updateConnectableStatus, "Update a connectable (branch or injection) status");
 
     py::enum_<element_type>(m, "ElementType")
+            .value("BUS", element_type::BUS)
             .value("LINE", element_type::LINE)
             .value("TWO_WINDINGS_TRANSFORMER", element_type::TWO_WINDINGS_TRANSFORMER)
+            .value("THREE_WINDINGS_TRANSFORMER", element_type::THREE_WINDINGS_TRANSFORMER)
             .value("GENERATOR", element_type::GENERATOR)
             .value("LOAD", element_type::LOAD)
+            .value("SHUNT_COMPENSATOR", element_type::SHUNT_COMPENSATOR)
+            .value("DANGLING_LINE", element_type::DANGLING_LINE)
+            .value("LCC_CONVERTER_STATION", element_type::LCC_CONVERTER_STATION)
+            .value("VSC_CONVERTER_STATION", element_type::VSC_CONVERTER_STATION)
+            .value("STATIC_VAR_COMPENSATOR", element_type::STATIC_VAR_COMPENSATOR)
             .export_values();
 
     m.def("get_network_elements_ids", &gridpy::getNetworkElementsIds, "Get network elements ids for a given element type",
           py::arg("network"), py::arg("element_type"), py::arg("nominal_voltages"),
-          py::arg("countries"), py::arg("main_connected_component"));
+          py::arg("countries"), py::arg("main_connected_component"), py::arg("main_synchronous_component"),
+          py::arg("not_connected_to_same_bus_at_both_sides"));
 
     m.def("load_network", &gridpy::loadNetwork, "Load a network from a file");
 
@@ -327,5 +337,30 @@ PYBIND11_MODULE(_gridpy, m) {
     m.def("get_reference_flows", &gridpy::getReferenceFlows, "Get sensitivity analysis result reference flows for a given contingency",
           py::arg("sensitivity_analysis_result_context"), py::arg("contingency_id"));
 
+    py::class_<series>(m, "Series")
+            .def_property_readonly("name", [](const series& s) {
+                return s.name;
+            })
+            .def_property_readonly("type", [](const series& s) {
+                return s.type;
+            })
+            .def_property_readonly("str_data", [](const series& s) {
+                return gridpy::toVector<std::string>((array*) &s.data);
+            })
+            .def_property_readonly("double_data", [](const series& s) {
+                return gridpy::toVector<double>((array*) &s.data);
+            })
+            .def_property_readonly("int_data", [](const series& s) {
+                return gridpy::toVector<int>((array*) &s.data);
+            })
+            .def_property_readonly("boolean_data", [](const series& s) {
+                return gridpy::toVector<bool>((array*) &s.data);
+            });
+
+    m.def("create_network_elements_series_array", &gridpy::createNetworkElementsSeriesArray, "Create a network elements series array for a given element type",
+          py::arg("network"), py::arg("element_type"));
+
+    bindArray<gridpy::SeriesArray>(m, "SeriesArray");
+
     m.def("destroy_object_handle", &gridpy::destroyObjectHandle, "Destroy Java object handle", py::arg("object_handle"));
 }
