@@ -80,6 +80,12 @@ Array<limit_violation>::~Array() {
     // already freed by contingency_result
 }
 
+template<>
+Array<series>::~Array() {
+    GraalVmGuard guard;
+    freeNetworkElementsSeriesArray(guard.thread(), delegate_);
+}
+
 template<typename T>
 class ToPtr {
 public:
@@ -121,7 +127,8 @@ public:
     }
 };
 
-std::vector<std::string> fromCharPtrPtr(array* arrayPtr) {
+template<>
+std::vector<std::string> toVector(array* arrayPtr) {
     std::vector<std::string> strings;
     strings.reserve(arrayPtr->length);
     for (int i = 0; i < arrayPtr->length; i++) {
@@ -131,6 +138,25 @@ std::vector<std::string> fromCharPtrPtr(array* arrayPtr) {
     return strings;
 }
 
+template<typename F, typename... ARGS>
+void handleException(F f, ARGS... args) {
+   exception_handler exc;
+   f(args..., &exc);
+   if (exc.message) {
+       throw GridPyError(exc.message);
+   }
+}
+
+template<typename T, typename F, typename... ARGS>
+T handleException(F f, ARGS... args) {
+   exception_handler exc;
+   auto r = f(args..., &exc);
+   if (exc.message) {
+       throw GridPyError(exc.message);
+   }
+   return r;
+}
+
 void setDebugMode(bool debug) {
     GraalVmGuard guard;
     setDebugMode(guard.thread(), debug);
@@ -138,79 +164,84 @@ void setDebugMode(bool debug) {
 
 std::string getVersionTable() {
     GraalVmGuard guard;
-    return std::string(getVersionTable(guard.thread()));
+    return std::string(handleException<char*>(::getVersionTable, guard.thread()));
 }
 
 void* createEmptyNetwork(const std::string& id) {
     GraalVmGuard guard;
-    return createEmptyNetwork(guard.thread(), (char*) id.data());
+    return handleException<void*>(::createEmptyNetwork, guard.thread(), (char *) id.data());
 }
 
 void* createIeee14Network() {
     GraalVmGuard guard;
-    return createIeee14Network(guard.thread());
+    return handleException<void*>(::createIeee14Network, guard.thread());
 }
 
 void* createEurostagTutorialExample1Network() {
     GraalVmGuard guard;
-    return createEurostagTutorialExample1Network(guard.thread());
+    return handleException<void*>(::createEurostagTutorialExample1Network, guard.thread());
 }
 
 void* loadNetwork(const std::string& file) {
     GraalVmGuard guard;
-    return loadNetwork(guard.thread(), (char*) file.data());
+    return handleException<void*>(::loadNetwork, guard.thread(), (char *) file.data());
 }
 
 void dumpNetwork(void* network, const std::string& file, const std::string& format) {
     GraalVmGuard guard;
-    dumpNetwork(guard.thread(), network, (char*) file.data(), (char*) format.data());
+    handleException(::dumpNetwork, guard.thread(), network, (char *) file.data(), (char *) format.data());
 }
 
 bool updateSwitchPosition(void* network, const std::string& id, bool open) {
     GraalVmGuard guard;
-    return updateSwitchPosition(guard.thread(), network, (char*) id.data(), open);
+    return handleException<bool>(::updateSwitchPosition, guard.thread(), network, (char *) id.data(), open);
 }
 
 bool updateConnectableStatus(void* network, const std::string& id, bool connected) {
     GraalVmGuard guard;
-    return updateConnectableStatus(guard.thread(), network, (char*) id.data(), connected);
+    return handleException<bool>(::updateConnectableStatus, guard.thread(), network, (char *) id.data(), connected);
 }
 
 std::vector<std::string> getNetworkElementsIds(void* network, element_type elementType, const std::vector<double>& nominalVoltages,
-                                               const std::vector<std::string>& countries, bool mainCc) {
+                                               const std::vector<std::string>& countries, bool mainCc, bool mainSc,
+                                               bool notConnectedToSameBusAtBothSides) {
     GraalVmGuard guard;
     ToDoublePtr nominalVoltagePtr(nominalVoltages);
     ToCharPtrPtr countryPtr(countries);
-    array* elementsIdsArrayPtr = getNetworkElementsIds(guard.thread(), network, elementType, nominalVoltagePtr.get(), nominalVoltages.size(),
-                                                       countryPtr.get(), countries.size(), mainCc);
-    std::vector<std::string> elementsIds = fromCharPtrPtr(elementsIdsArrayPtr);
+    auto elementsIdsArrayPtr = handleException<array*>(::getNetworkElementsIds, guard.thread(), network, elementType,
+                                                       nominalVoltagePtr.get(), nominalVoltages.size(),
+                                                       countryPtr.get(), countries.size(), mainCc, mainSc,
+                                                       notConnectedToSameBusAtBothSides);
+    std::vector<std::string> elementsIds = toVector<std::string>(elementsIdsArrayPtr);
     freeNetworkElementsIds(guard.thread(), elementsIdsArrayPtr);
     return elementsIds;
 }
 
 LoadFlowComponentResultArray* runLoadFlow(void* network, bool dc, load_flow_parameters& parameters) {
     GraalVmGuard guard;
-    return new LoadFlowComponentResultArray(runLoadFlow(guard.thread(), network, dc, &parameters));
+    return new LoadFlowComponentResultArray(
+            handleException<array*>(::runLoadFlow, guard.thread(), network, dc, &parameters));
 }
 
 BusArray* getBusArray(void* network) {
     GraalVmGuard guard;
-    return new BusArray(getBusArray(guard.thread(), network));
+    return new BusArray(handleException<array*>(::getBusArray, guard.thread(), network));
 }
 
 GeneratorArray* getGeneratorArray(void* network) {
     GraalVmGuard guard;
-    return new GeneratorArray(getGeneratorArray(guard.thread(), network));
+    return new GeneratorArray(handleException<array*>(::getGeneratorArray, guard.thread(), network));
 }
 
 LoadArray* getLoadArray(void* network) {
     GraalVmGuard guard;
-    return new LoadArray(getLoadArray(guard.thread(), network));
+    return new LoadArray(handleException<array*>(::getLoadArray, guard.thread(), network));
 }
 
 void writeSingleLineDiagramSvg(void* network, const std::string& containerId, const std::string& svgFile) {
     GraalVmGuard guard;
-    writeSingleLineDiagramSvg(guard.thread(), network, (char*) containerId.data(), (char*) svgFile.data());
+    handleException(::writeSingleLineDiagramSvg, guard.thread(), network, (char *) containerId.data(),
+                    (char *) svgFile.data());
 }
 
 void* createSecurityAnalysis() {
@@ -221,12 +252,14 @@ void* createSecurityAnalysis() {
 void addContingency(void* analysisContext, const std::string& contingencyId, const std::vector<std::string>& elementsIds) {
     GraalVmGuard guard;
     ToCharPtrPtr elementIdPtr(elementsIds);
-    addContingency(guard.thread(), analysisContext, (char*) contingencyId.data(), elementIdPtr.get(), elementsIds.size());
+    handleException(::addContingency, guard.thread(), analysisContext, (char *) contingencyId.data(), elementIdPtr.get(),
+                    elementsIds.size());
 }
 
 ContingencyResultArray* runSecurityAnalysis(void* securityAnalysisContext, void* network, load_flow_parameters& parameters) {
     GraalVmGuard guard;
-    return new ContingencyResultArray(runSecurityAnalysis(guard.thread(), securityAnalysisContext, network, &parameters));
+    return new ContingencyResultArray(
+            handleException<array*>(::runSecurityAnalysis, guard.thread(), securityAnalysisContext, network, &parameters));
 }
 
 void* createSensitivityAnalysis() {
@@ -238,23 +271,30 @@ void setFactorMatrix(void* sensitivityAnalysisContext, const std::vector<std::st
     GraalVmGuard guard;
     ToCharPtrPtr branchIdPtr(branchesIds);
     ToCharPtrPtr injectionOrTransfoIdPtr(injectionsOrTransfosIds);
-    setFactorMatrix(guard.thread(), sensitivityAnalysisContext, branchIdPtr.get(), branchesIds.size(),
+    handleException(::setFactorMatrix, guard.thread(), sensitivityAnalysisContext, branchIdPtr.get(), branchesIds.size(),
                     injectionOrTransfoIdPtr.get(), injectionsOrTransfosIds.size());
 }
 
 void* runSensitivityAnalysis(void* sensitivityAnalysisContext, void* network, load_flow_parameters& parameters) {
     GraalVmGuard guard;
-    return runSensitivityAnalysis(guard.thread(), sensitivityAnalysisContext, network, &parameters);
+    return handleException<void*>(::runSensitivityAnalysis, guard.thread(), sensitivityAnalysisContext, network, &parameters);
 }
 
 matrix* getSensitivityMatrix(void* sensitivityAnalysisResultContext, const std::string& contingencyId) {
     GraalVmGuard guard;
-    return getSensitivityMatrix(guard.thread(), sensitivityAnalysisResultContext, (char*) contingencyId.c_str());
+    return handleException<matrix*>(::getSensitivityMatrix, guard.thread(), sensitivityAnalysisResultContext,
+                                    (char *) contingencyId.c_str());
 }
 
 matrix* getReferenceFlows(void* sensitivityAnalysisResultContext, const std::string& contingencyId) {
     GraalVmGuard guard;
-    return getReferenceFlows(guard.thread(), sensitivityAnalysisResultContext, (char*) contingencyId.c_str());
+    return handleException<matrix*>(::getReferenceFlows, guard.thread(), sensitivityAnalysisResultContext,
+                                    (char *) contingencyId.c_str());
+}
+
+SeriesArray* createNetworkElementsSeriesArray(void* network, element_type elementType) {
+    GraalVmGuard guard;
+    return new SeriesArray(handleException<array*>(::createNetworkElementsSeriesArray, guard.thread(), network, elementType));
 }
 
 void destroyObjectHandle(void* objectHandle) {
