@@ -9,10 +9,18 @@
 
 #include <string>
 #include <vector>
+#include <stdexcept>
 #include "gridpy-api.h"
 
 namespace gridpy {
 
+class GridPyError : public std::runtime_error {
+public:
+    GridPyError(const char* msg)
+        : runtime_error(msg) {
+    }
+};
+
 template<typename T>
 class Array {
 public:
@@ -38,6 +46,21 @@ typedef Array<generator> GeneratorArray;
 typedef Array<load> LoadArray;
 typedef Array<contingency_result> ContingencyResultArray;
 typedef Array<limit_violation> LimitViolationArray;
+typedef Array<series> SeriesArray;
+
+template<typename T>
+std::vector<T> toVector(array* arrayPtr) {
+    std::vector<T> values;
+    values.reserve(arrayPtr->length);
+    for (int i = 0; i < arrayPtr->length; i++) {
+        T value = *((T*) arrayPtr->ptr + i);
+        values.push_back(value);
+    }
+    return values;
+}
+
+template<>
+std::vector<std::string> toVector(array* arrayPtr);
 
 enum LoadFlowComponentStatus {
     CONVERGED = 0,
@@ -88,7 +111,8 @@ bool updateSwitchPosition(void* network, const std::string& id, bool open);
 bool updateConnectableStatus(void* network, const std::string& id, bool connected);
 
 std::vector<std::string> getNetworkElementsIds(void* network, element_type elementType, const std::vector<double>& nominalVoltages,
-                                               const std::vector<std::string>& countries, bool mainCc);
+                                               const std::vector<std::string>& countries, bool mainCc, bool mainSc,
+                                               bool notConnectedToSameBusAtBothSides);
 
 void* loadNetwork(const std::string& file);
 
@@ -120,6 +144,8 @@ matrix* getSensitivityMatrix(void* sensitivityAnalysisResultContext, const std::
 
 matrix* getReferenceFlows(void* sensitivityAnalysisResultContext, const std::string& contingencyId);
 
+SeriesArray* createNetworkElementsSeriesArray(void* network, element_type elementType);
+
 void destroyObjectHandle(void* objectHandle);
 
 }
